# {{ ansible_managed }}

{% raw %}
{% extends parent_template %}
{% endraw %}

{% for block_name, block_contents in kolla_build_blocks.items() %}
{{ '{% block ' ~ block_name ~ ' %}' }}
{{ block_contents }}
{{ '{% endblock %}' }}

{% endfor %}

{% for var_name, var_value in kolla_build_customizations.items() %}
{{ '{% set ' ~ var_name ~ ' = ' ~ var_value | to_json ~ ' %}' }}

{% endfor %}
