{
{%- if docker_registry_mirrors | length > 0 %}
  "registry-mirrors": [
{%- for mirror in docker_registry_mirrors %}
      "{{ mirror }}"{%- if not loop.last -%},{%- endif %}
{%- endfor %}
  ],
{%- endif %}
{% if docker_daemon_mtu %}
  "mtu": {{ docker_daemon_mtu }},
{% endif %}
{% if docker_daemon_live_restore %}
  "live-restore": {{ docker_daemon_live_restore | to_json }},
{% endif %}
  "storage-opts": [
{% if docker_storage_driver == 'devicemapper' %}
    "dm.thinpooldev=/dev/mapper/{{ docker_storage_volume_group | replace('-', '--') }}-{{ docker_storage_volume_thinpool | replace('-', '--') }}",
    "dm.use_deferred_removal=true",
    "dm.use_deferred_deletion=true"
{% endif %}
  ]
}
