---
# Don't build an IPA deployment image, instead download upstream images.
create_ipa_image: false
download_ipa: true

# Don't build a disk image. It takes time and can be unreliable.
use_cirros: true
