# Simple inventory for bootstrapping Kolla seed node.
[seed]
{% for seed in groups.get('seed', []) %}
{{ seed }}
{% endfor %}

[seed:vars]
ansible_user={{ kolla_ansible_user }}
{% if kolla_ansible_target_venv is not none %}
# Execute ansible modules on the remote target hosts using a virtualenv.
ansible_python_interpreter={{ kolla_ansible_target_venv }}/bin/python
{% endif %}

[baremetal:children]
seed

[bifrost:children]
seed
