---
# NOTE(mgoddard): Don't reboot after disabling SELinux during CI testing, as
# Ansible is run directly on the controller.
disable_selinux_do_reboot: false

# Use the OpenStack infra's Dockerhub mirror.
docker_registry_mirrors:
  - "http://{{ zuul_site_mirror_fqdn }}:8082/"

kolla_docker_namespace: "openstack.kolla"
# use the published images from a site mirror of quay.io
kolla_docker_registry: "{{ zuul_site_mirror_fqdn }}:4447"
kolla_source_url: "{{ ansible_env.PWD ~ '/' ~ zuul.projects['opendev.org/openstack/kolla'].src_dir }}"
kolla_source_version: "{{ zuul.projects['opendev.org/openstack/kolla'].checkout }}"
kolla_ansible_source_url: "{{ ansible_env.PWD ~ '/' ~ zuul.projects['opendev.org/openstack/kolla-ansible'].src_dir }}"
kolla_ansible_source_version: "{{ zuul.projects['opendev.org/openstack/kolla-ansible'].checkout }}"
kolla_openstack_logging_debug: True
pip_upper_constraints_file: "/tmp/upper-constraints.txt"

# Use the CI infra's PyPI mirror.
pip_local_mirror: true
pip_index_url: "http://{{ zuul_site_mirror_fqdn }}/pypi/simple"
pip_trusted_hosts:
  - "{{ zuul_site_mirror_fqdn }}"

# NOTE(mgoddard): CentOS 8 removes interfaces from their bridge during ifdown,
# and removes the bridge if there are no interfaces left. When Kayobe bounces
# veth links plugged into the bridge, it causes the bridge which has the IP we
# are using for SSH to be removed. Use a dummy interface.
aio_bridge_ports:
  - dummy1

# Enable ironic for testing baremetal compute.
kolla_enable_ironic: true

{% if tls_enabled %}
kolla_enable_tls_external: "yes"
kolla_enable_tls_internal: "yes"

# FIXME: ipa-images fails to access OS_CACERT from /home/zuul.
kayobe_ansible_user: zuul

kolla_ironic_pxe_append_params_extra:
  - ipa-insecure=1
{% endif %}
